/*******************************************************************************
*
* Copyright [2014-]2014 Freescale Semiconductor, Inc.

*
* This software is owned or controlled by Freescale Semiconductor.
* Use of this software is governed by the Freescale License
* distributed with this Material.
* See the LICENSE file distributed for more details.
*
*
*******************************************************************************/

#include <stdio.h>
#include "board.h"
#include "fsl_os_abstraction.h"
#include "fsl_hwtimer.h"
#include "fsl_debug_console.h"
#include "fsl_interrupt_manager.h"
#include "fsl_clock_manager.h"
#include "fsl_gpio_driver.h"
#include "fsl_pwm_driver.h"
#include "fsl_cadc_driver.h"
#include "fsl_uart_driver.h"
#include "fsl_dac_driver.h"
#include "fsl_port_hal.h"
#include "fsl_dac_hal.h"
#include "fsl_cadc_hal.h"
#include "math.h"
#include "hardware.h"


/*******************************************************************************
* Variables
******************************************************************************/
extern const hwtimer_devif_t kSystickDevif;
extern const hwtimer_devif_t kPitDevif;
hwtimer_t hwtimer;
#define HWTIMER_LL_DEVIF    kSystickDevif


volatile float x_command = 0, x_error = 0, x_pos = 0, x_set = 0, P_x = 0, I_x = 0, D_x = 0, x_prev = 0;
volatile float y_command = 0, y_error = 0, y_pos = 0, y_set = 0, P_y = 0, I_y = 0, D_y = 0, y_prev = 0;
volatile float P = 01;
volatile float I = 0.01;
volatile float D = 10;

//void hwtimer_callback(void* data);

void hwtimer_callback(void* data)
{
	SET_TRIG();
	Convert_CADC();

	//read current position
	x_pos = Read_CADC(GALVO_POS_Y);
	y_pos = Read_CADC(GALVO_POS_X);
	//read setpoint
	x_set = Read_CADC(GALVO_SET_X);
	y_set = Read_CADC(GALVO_SET_Y);


	x_prev = x_error;
	y_prev = y_error;

	x_error = x_pos - x_set;
	y_error = y_pos - y_set;

	P_x = x_error * P;
	P_y = y_error * P;

	I_x = I_x + x_error * I;
	I_y = I_y + y_error * I;

	D_x = (x_error - x_prev) * D;
	D_y = (y_error - y_prev) * D;

	x_command = P_x + I_x + D_x;
	y_command = P_y + I_y + D_y;

	//limit output values
	if (x_command > 1.0)
		x_command = 1;
	if (x_command < -1.0)
		x_command = -1;

	if (y_command > 1.0)
		y_command = 1;
	if (y_command < -1.0)
		y_command = -1;

	//update PWM generators
	Galvo_Update(GALVO_CON_X, x_command);
	Galvo_Update(GALVO_CON_Y, y_command);
	RESET_TRIG();
}



int main(void)
{
 //   uint16_t i;
  //  uint8_t channel;
	float D_set, I_set, P_set;
    /* Init hardware, OSA, dbg console */
    hardware_init();
    OSA_Init();
    PWM_DRV_Init(0U);
    PWM_Init();
    ADC_init();
    GPIO_Init();
    PID_INIT();
    PRINTF("Project131\r\n");


	   // Hwtimer initialization
	    if (kHwtimerSuccess != HWTIMER_SYS_Init(&hwtimer, &HWTIMER_LL_DEVIF, HWTIMER_LL_ID, NULL))
	    {
	        PRINTF("\r\nError: hwtimer initialization.\r\n");
	    }

	    /*
	     * In case you wish to raise the Systick ISR priority, then use the below command with the
	     * appropriate priority setting:
	     * NVIC_SetPriority(SysTick_IRQn, isrPrior);
	     */
	    if (kHwtimerSuccess != HWTIMER_SYS_SetPeriod(&hwtimer, HWTIMER_PERIOD))
	    {
	        PRINTF("\r\nError: hwtimer set period.\r\n");
	    }
	    if (kHwtimerSuccess != HWTIMER_SYS_RegisterCallback(&hwtimer, hwtimer_callback, NULL))
	    {
	        PRINTF("\r\nError: hwtimer callback registration.\r\n");
	    }
	    if (kHwtimerSuccess != HWTIMER_SYS_Start(&hwtimer))
	    {
	        PRINTF("\r\nError: hwtimer start.\r\n");
	    }

   // 	OSA_TimeDelay(100);
    //	P = 0.5;
    //	OSA_TimeDelay(100);
    //	P = 1;

    while(1)
    {
    	//SET_TRIG();
    	PRINTF("%01.04f \t %01.04f \t %01.04f\t\t", x_set, x_pos, x_command+1.0);  //printf seems borked for negative values.
    	PRINTF("%01.04f \t %01.04f \t %01.04f\r\n", y_set, y_pos, y_command+1.0);

    	PRINTF("enter P I D\r\n");
        debug_scanf("%f %f %f",&P_set, &I_set, &D_set);
        debug_printf("%f\n",D_set);

    	D = D_set;
    	I = I_set;
    	P = P_set;
    	PRINTF("%01.04f \t %01.04f \t %01.04f\r\n", P, I,D);
    	//RESET_TRIG();

    }

    while(true)
    {
    	SET_SCOPE();
    	RESET_SCOPE();
    }
}

